package tags.jdbc;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import java.util.StringTokenizer;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;

import beans.jdbc.DbConnectionPool;
import beans.util.ResourceException;

public class PrepareCallTag extends BodyTagSupport {
   private String id, scope = "page";

   public void setId(String id) {
      this.id = id;
   }
   public void setScope(String scope) {
      this.scope = scope;
   }
   public int doEndTag() throws JspException {
      DbConnectionPool pool = (DbConnectionPool)
                              pageContext.getServletContext().
                              getAttribute("db-connection-pool");
      Connection con;
      try {
         con = (Connection)pool.getResource();
      }
      catch(ResourceException ex) {
         throw new JspException(ex.getMessage());
      }
      storeProcedure(con); // store procedure in database
      pageContext.setAttribute(id, prepareCall(con),
                               getConstantForScope(scope));
      return EVAL_PAGE;
   }
   private void storeProcedure(Connection con) 
                                             throws JspException {
      try {
         Statement st = con.createStatement();
         st.executeUpdate(bodyContent.getString());
      }
      catch(SQLException ex) {
         throw new JspException(ex.getMessage());
      }
   }
   private CallableStatement prepareCall(Connection con)
                                             throws JspException {
      CallableStatement cstate = null;
      try {
         cstate = con.prepareCall(
                           "{call " + getProcedureName() + "}",
                           ResultSet.TYPE_SCROLL_INSENSITIVE,
                           ResultSet.CONCUR_READ_ONLY);
      }
      catch(SQLException ex) {
         throw new JspException(ex.getMessage());
      }
      return cstate;
   }
   private String getProcedureName() {
      StringTokenizer tok = new StringTokenizer(
                                bodyContent.getString());
      String procedureName = null;
      boolean createFound = false, procedureFound = false;;

      while(tok.hasMoreElements()) {
         String token = tok.nextToken();

         if(token.equals("create")) {
            createFound = true;
            continue;
         }
         if(createFound && token.equals("procedure")) {
            procedureFound = true;
            continue;
         }
         if(createFound && procedureFound) {  
            procedureName = token;
            break;
         }
      }
      return procedureName;
   }
   private int getConstantForScope(String scope) {
      int constant = PageContext.PAGE_SCOPE;

      if(scope.equalsIgnoreCase("page")) 
         constant = PageContext.PAGE_SCOPE;
      else if(scope.equalsIgnoreCase("request")) 
         constant = PageContext.REQUEST_SCOPE;
      else if(scope.equalsIgnoreCase("session")) 
         constant = PageContext.SESSION_SCOPE;
      else if(scope.equalsIgnoreCase("application")) 
         constant = PageContext.APPLICATION_SCOPE;

      return constant;
   }
}
