package tags.i18n;

import java.text.MessageFormat;
import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.tagext.*;
import beans.i18n.*;

import beans.app.Constants;

public class MessageTag extends TagSupport 
					      implements beans.app.Constants {
   protected static BundleCache bundles = new BundleCache();

   protected String     base = null, key = null;
   protected Locale   locale = null;
   protected Object[] params = null;

   public void setBase(String base)       { this.base = base; }
   public void setKey(String key)         { this.key = key; }
   public void setLocale(Locale locale)   { this.locale = locale;}
   public void setParams(Object[] params) { this.params = params;}

   public int doEndTag() throws JspException {
		base = (base != null) ? base : DEFAULT_I18N_BASE;
      String message = getMessage();

      if(params != null)
			message = formatWithParams(message);

      try {
      	pageContext.getOut().write(message);
      }
      catch(Exception ex) { // IO or MissingResource
        	throw new JspException(ex.getMessage());
      }

      return EVAL_PAGE;
   }
   protected String getMessage() throws JspException {
		String msg = null;

      if(locale != null)
         msg = bundles.getString(base, locale, key);

      if(msg == null)
         msg = getMessageUsingSessionLocale();

      if(msg == null)
         msg = getMessageUsingBrowserLocales();

		if(msg == null) // for goodness sake!
			throw new JspException("can't process message");

		return msg;
	}
	protected String formatWithParams(String message) {
      MessageFormat fmt = new MessageFormat(message);

   	if(locale != null) {
   		fmt.setLocale(locale);
   		fmt.applyPattern(message);
   	}
   	return fmt.format(params);
	}
   protected String getMessageUsingBrowserLocales() {
      String message = null;
      LocaleDetector locale_detector = new LocaleDetector();
      Vector locales = locale_detector.detectLocales(
                     (HttpServletRequest)pageContext.getRequest(),
                     pageContext.getResponse());
      ResourceBundle bundle = (new BundleLocator()).locateBundle(
                                                   base, locales);
      if(bundle != null) {
         bundles.addBundle(bundle, base);
         message = bundle.getString(key);
      }
      return message;
   }
   protected String getMessageUsingSessionLocale() {
      String message = null;
      HttpSession session = pageContext.getSession();
      Locale locale = (Locale)session.getAttribute(LOCALE_KEY);

      if(locale != null) {
         Vector locales = new Vector();
         locales.add(locale);

         ResourceBundle bundle = (new BundleLocator()).
                                      locateBundle(base, locales);
         if(bundle != null) {
              bundles.addBundle(bundle, base);
              message = bundle.getString(key);
         }
      }
      return message;
   }

   // prepare this tag handler for reuse

   public void release() {
      base = key = null;
      locale = null;
      params = null;
   }
}
