package tags.jdbc;

import java.sql.*;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;
import tags.jdbc.beans.Query;

public class ExecutePreparedStatementTag extends TagSupport {
   private Object[] variables = null;
   private String scope = "page";

   public void setId(String id) {
      this.id = id;
   }
   public void setScope(String scope) {
      this.scope = scope;
   }
   public void setVariables(Object[] variables) {
      this.variables = variables;
   }
   public int doEndTag() throws JspException {
      PreparedStatement ps = (PreparedStatement)
                              pageContext.findAttribute(id);
      if(ps == null) {
         throw new JspException("No prepared statement " + id);
      }
      try {
         setVariables(ps, variables);
         ResultSet rs = ps.executeQuery();

         Query query = new Query(ps, rs);
         Query.save(query, pageContext, id, scope);
      }
      catch(SQLException sqlex) {
         throw new JspException(sqlex.getMessage());
      }
      return EVAL_PAGE;
   }
   private void setVariables(PreparedStatement ps, 
                              Object[] variables) 
                              throws SQLException, JspException {
      for(int i=0; i < variables.length; ++i) {
         Object object = variables[i];

         if(object instanceof String) {
            ps.setString(i+1, (String)object);
         }
         if(object instanceof Integer) {
            int value = ((Integer)object).intValue();
            ps.setInt(i+1, value);
         }
         if(object instanceof Double) {
            double value = ((Double)object).doubleValue();
            ps.setDouble(i+1, value);
         }
         // Add more code here for other types of variables,
         // such as shorts, times, timestamps, etc. See
         // java.sql.PreparedStatement for valid types.
         else {
            throw new JspException("ExecuteStatementTag:" +
               "setVariables(): unkown parameter type");
         }
      }
   }
}
