package tags.jdbc;

import java.sql.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.StringTokenizer;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.BodyTagSupport;
import beans.jdbc.ConnectionException;
import beans.jdbc.DbConnectionPool;

public class TransactionTag extends BodyTagSupport 
								 implements Constants {
   private String file;

   public void setFile(String file) {
      this.file = file;
   }
   public int doEndTag() throws JspException {
      Connection con = getConnection();
      String stmts = null;
   
      if(file != null) stmts = getStatementsFromFile();   
      else             stmts = bodyContent.getString();

      try {
         executeTransaction(con, stmts);
      }
      catch(SQLException ex) {
         throw new JspException(ex.getMessage());
      }
      return EVAL_PAGE;
   }
   private Connection getConnection() throws JspException {
      DbConnectionPool pool = (DbConnectionPool)
                              pageContext.getServletContext().
                              getAttribute(DBPOOL_KEY);
      Connection con;
      try {
         con = (Connection)pool.getConnection();
      }
      catch(ConnectionException ex) {
         throw new JspException(ex.getMessage());
      }
      return con;
   }
   private String getStatementsFromFile() throws JspException {
      BufferedReader reader = new BufferedReader(
                              new InputStreamReader(
                                 pageContext.getServletContext().
                                 getResourceAsStream(file)));
      String stmts = "", line = null;

      try {
         while((line = reader.readLine()) != null) {
            stmts += line;
         }
      }
      catch(java.io.IOException ex) {
         throw new JspException(ex.getMessage());
      }
      return stmts;
   }
   private void executeTransaction(Connection con, String stmts) 
                                             throws SQLException {
      boolean wasAutoCommit = con.getAutoCommit();

      try { 
         StringTokenizer tok = new StringTokenizer(stmts, ";");

         con.setAutoCommit(false);

         while(tok.hasMoreElements()) {
            Statement stmt = con.createStatement();
            String query =(String)tok.nextElement(); 
            stmt.execute(query);
            stmt.close();
         }
      }
      catch(Exception ex) {
         con.rollback();
      }
      finally { 
         con.commit();
      }
      con.setAutoCommit(wasAutoCommit); 
   }
}
