package beans.regions;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;

// A section is content with a name that implements 
// Content.render. That method renders content either by including
// it or by printing it directly, depending upon the direct 
// value passed to the Section constructor. 
//
// Note that a section's content can also be a region;if so, 
// Region.render is called from Section.Render().

public class Section extends Content {
   protected final String name;

   public Section(String name, String content, String direct) {
      super(content, direct);
      this.name = name;
   }
   public String getName() { 
      return name;    
   }
   public void render(PageContext pageContext) 
                                  throws JspException {
      if(content != null) {
         // see if this section's content is a region
         Region region = (Region)pageContext.
                                 findAttribute(content);
         if(region != null) {
            // render the content as a region
            RegionStack.push(pageContext, region);
            region.render(pageContext);
            RegionStack.pop(pageContext);
         }
         else {
            if(isDirect()) {
               try {
                  pageContext.getOut().print(content.toString());
               }
               catch(java.io.IOException ex) {
                  throw new JspException(ex.getMessage());
               }
            }
            else {
                 try {
                  pageContext.include(content.toString());
               }
               catch(Exception ex) { 
                    throw new JspException(ex.getMessage());
               }
            }
         }
      }
   }
   public String toString() { 
      return "Section: " + name + ", content= " + 
                                     content.toString();
   }
}
