import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import actions.*;
import actions.events.ActionEvent;

public class ActionServlet extends HttpServlet {
   private ActionFactory factory = new ActionFactory();
   private StateManager stateManager = new StateManager();

   public void init(ServletConfig config) throws ServletException{
      super.init(config);

      ResourceBundle bundle = null;

      try {
         bundle = ResourceBundle.getBundle(
                  config.getInitParameter("action-mappings"));
      }
      catch(MissingResourceException e) {
         throw new ServletException(e);
      }
      getServletContext().setAttribute("action-mappings", bundle);
   }
   public void service(HttpServletRequest req,
                       HttpServletResponse res)
                         throws ServletException {
      Action action = getAction(req);

      action.beforeAction(new ActionEvent(action, 
                        ActionEvent.BEFORE_ACTION,
                        req, res));

      ActionRouter router = performAction(action, req, res);

      action.afterAction(new ActionEvent(action, 
                       ActionEvent.AFTER_ACTION,
                       req, res));

      // routers could fire events in a manner similar to actions
      routeAction(router, req, res);
   }
   protected Action getAction(HttpServletRequest req)
                                        throws ServletException {
      Action action = null;
      try {
         action = factory.getAction(getActionClass(req),
                                    getClass().getClassLoader());
         //action.addActionListener(stateManager);
      }
      catch(Exception ex) {
         throw new ServletException(ex);
      }
      return action;
   }
   protected ActionRouter performAction(Action action, 
                                        HttpServletRequest req,
                                        HttpServletResponse res) 
         										 throws ServletException {
      ActionRouter router = null;
      try {
         router = action.perform(this,req,res);
      }
      catch(Exception ex) {
         throw new ServletException(ex);
      }
      return router;
   }
   protected void routeAction(ActionRouter router,
                              HttpServletRequest req,
                                HttpServletResponse res)
                                throws ServletException {
      try {
         router.route(this, req, res);
      }
      catch(Exception ex) {
         throw new ServletException(ex);
      }
   }
   private String getClassname(HttpServletRequest req) {
      String path = req.getServletPath();
      int slash = path.lastIndexOf("/"), 
          period = path.lastIndexOf(".");

      if(period > 0 && period > slash)
       path = path.substring(slash+1, period);

      return path;
   }
   private String getActionClass(HttpServletRequest req) {
      ResourceBundle bundle = (ResourceBundle)getServletContext().
                              getAttribute("action-mappings");
      return (String)bundle.getObject(getActionKey(req));
   }
   private String getActionKey(HttpServletRequest req) {
      String path = req.getServletPath();
      int slash = path.lastIndexOf("/"), 
          period = path.lastIndexOf(".");

      if(period > 0 && period > slash)
       path = path.substring(slash+1, period);

      return path;
   }
}
