package actions;

import java.util.Enumeration;
import java.util.Vector;
import javax.servlet.ServletException;
import actions.events.ActionEvent;
import actions.events.ActionListener;

public abstract class ActionBase implements Action, 
                                            ActionListener {
   private Vector listeners = new Vector();

   public void addActionListener(ActionListener listener) {
      listeners.addElement(listener);   
   }
   public void removeActionListener(ActionListener listener) {
      listeners.remove(listener);   
   }
   public boolean isSensitive() { // override this method 
      return false;               // if this action is sensitive
   }
   public boolean hasSensitiveForms() { // override this method 
      return false;               // if you have sensitive content
   }
   public void beforeAction(ActionEvent event) 
                                        throws ServletException {
      fireEvent(event);
   }
   public void afterAction(ActionEvent event)
                                        throws ServletException {
      fireEvent(event);
   } 
   protected void fireEvent(ActionEvent event) 
                                       throws ServletException {
      Enumeration it = listeners.elements();

      while(it.hasMoreElements()) {
         ActionListener listener = 
                        (ActionListener)it.nextElement();

         switch(event.getEventType()) {
            case ActionEvent.BEFORE_ACTION: 
                         listener.beforeAction(event);
                         break;
            case ActionEvent.AFTER_ACTION: 
                     listener.afterAction(event);
                     break;
         }
      }
   }
}
