package beans.i18n;

import java.util.Enumeration;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.servlet.ServletRequest;
import javax.servlet.jsp.JspException;

public class BundleLocator {
   public ResourceBundle locateBundle(ServletRequest request,
												  String base) 
												  throws JspException {
      Enumeration en = request.getLocales();
      Locale defaultLocale = Locale.getDefault();
      ResourceBundle fallbackBundle = null;
   
      try {
         fallbackBundle = ResourceBundle.getBundle(base, 
                                                  defaultLocale);
      }
      catch(MissingResourceException ex) {
			throw new JspException(ex.getMessage());
      }

      while(en.hasMoreElements()) {
         Locale locale = (Locale)en.nextElement();
         ResourceBundle bundle = null;

         try {
            bundle = ResourceBundle.getBundle(base, locale);
         }
         catch(MissingResourceException ex2) {
            // ignore missing bundles ...   
            continue;
         }
         if(bundle != fallbackBundle)
            return bundle;

         if(fallbackBundle != null && bundle == fallbackBundle) {
            String lang = locale.getLanguage();
            String defaultLanguage = defaultLocale.getLanguage();

            if(lang.equals(defaultLanguage))
               return bundle;
         }
      }
      return null;
   }
}
