package tags.i18n;

import java.text.*;
import java.util.Date;
import java.util.Locale;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.*;

public class FormatTag extends TagSupport {
   private static DateFormat dateFormat;
   private static NumberFormat numberFormat;
   private static final int UNASSIGNED=-1, 
                            TIME=0, DATE=1, DATE_TIME=2,
                            NUMBER=3, CURRENCY=4, PERCENT=5;
   private Date date;
   private double number = (double)UNASSIGNED;

   private Locale locale = null;

   private int dateStyle = UNASSIGNED, 
               timeStyle = UNASSIGNED,
             formatStyle = UNASSIGNED,
             numberStyle = UNASSIGNED; 

   // doEndTag()

   public int doEndTag() throws JspException {
      if(date != null) { // date attribute was set
         if(formatStyle == TIME)           processTime();
         else if(formatStyle == DATE)      processDate();
         else if(formatStyle == DATE_TIME) processDateTime();

         showFormatted(dateFormat.format(date));
      }
      if(number != UNASSIGNED) { // number attribute was set
         if(numberStyle == NUMBER)        processNumber();
         else if(numberStyle == CURRENCY) processCurrency();
         else if(numberStyle == PERCENT)  processPercent();

         showFormatted(numberFormat.format(number));
      }
      return EVAL_PAGE;
   }
   private void showFormatted(String string) throws JspException {
      try {
         pageContext.getOut().write(string);
      }
      catch(Exception ex) { 
         throw new JspException(ex.getMessage());
      }
   }

   // Attribute Setter Methods:

   public void setLocale(Locale locale){ this.locale = locale; }
   public void setDateStyle(int style) { this.dateStyle = style; }
   public void setTimeStyle(int style) { this.timeStyle = style; }

   public void setNumber(Double number){ 
      this.number = number.doubleValue(); numberStyle = NUMBER;
   } 
   public void setCurrency(Double number) { 
      setNumber(number); numberStyle = CURRENCY;
   }
   public void setPercent(Double number) { 
      setNumber(number); numberStyle = PERCENT;
   }
   public void setDate(Date date) { 
      this.date = date; formatStyle = DATE;
   }
   public void setDateTime(Date date) { 
      this.date = date; formatStyle = DATE_TIME; 
   }
   public void setTime(Date date) { 
      this.date = date; formatStyle = TIME; 
   }

   // Processing Methods

   private void processNumber() {
      numberFormat = (locale == null) ?
                     NumberFormat.getNumberInstance() :
                     NumberFormat.getNumberInstance(locale);
   }
   private void processCurrency() {
      numberFormat = (locale == null) ?
                     NumberFormat.getCurrencyInstance() :
                     NumberFormat.getCurrencyInstance(locale);
   }
   private void processPercent() {
      numberFormat = (locale == null) ?
                     NumberFormat.getPercentInstance() :
                     NumberFormat.getPercentInstance(locale);
   }
   private void processTime() {
      if(timeStyle == UNASSIGNED) {
         dateFormat = DateFormat.getTimeInstance();
      }
      else {
         dateFormat = (locale == null) ?
                     DateFormat.getTimeInstance(timeStyle) :
                     DateFormat.getTimeInstance(timeStyle,locale);
      }
   }
   private void processDate() {
      if(dateStyle == UNASSIGNED) {
         dateFormat = DateFormat.getDateInstance();
      }
      else {
         dateFormat = (locale == null) ?
                     DateFormat.getDateInstance(dateStyle) :
                     DateFormat.getDateInstance(dateStyle,locale);
      }
   }
   private void processDateTime() throws JspException {
      if(timeStyle == UNASSIGNED && dateStyle == UNASSIGNED) {
         dateFormat = DateFormat.getDateTimeInstance();
      }
      else {
         if(timeStyle == UNASSIGNED || dateStyle == UNASSIGNED) {
            throw new JspException("You must assign both" +
                                   "timeStyle and dateStyle, or" +
                                   " neither");
         }
         dateFormat = (locale == null) ?
                     DateFormat.getDateTimeInstance(
                                    timeStyle, dateStyle) :
                     DateFormat.getDateTimeInstance(
                                    timeStyle, dateStyle, locale);
      }
   }

   // prepare this tag handler for reuse

   public void release() {
      numberFormat = null;
      dateFormat = null;

      date = null;
      number = (double)UNASSIGNED;
      locale = null;

      dateStyle = timeStyle = 
      formatStyle = numberStyle = UNASSIGNED;
   }
}
