package tags.i18n;

import java.text.MessageFormat;

import java.util.Locale;
import java.util.ResourceBundle;

import javax.servlet.jsp.JspException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.tagext.TagSupport;

import beans.i18n.BundleLocator;
import beans.i18n.BundleCache;

public class MessageTag extends TagSupport {
   private static BundleCache bundles = new BundleCache();

   private String base, key;
   private Locale locale;
   private Object[] params;

   public void setBase(String base)     { this.base = base; }
   public void setLocale(Locale locale) { this.locale = locale; }
   public void setKey(String key)       { this.key = key; }
   public void setParams(Object[] params) { this.params = params;}

   public int doEndTag() throws JspException {
      MessageFormat fmt = null;
      String message = null;
   
      if(locale != null)
         message = bundles.getString(base, locale, key);

      if(message == null)
         message = getMessageUsingBrowserLocales();

      if(params != null) {
         fmt = new MessageFormat(message);

         if(locale != null) {
            fmt.setLocale(locale);
            fmt.applyPattern(message);
         }
         message = fmt.format(params);
      }
      try {
         pageContext.getOut().write(message);
      }
      catch(Exception ex) { // IO or MissingResource
         throw new JspException(ex.getMessage());
      }
      return EVAL_PAGE;
   }
   private String getMessageUsingBrowserLocales() 
                                             throws JspException {
      String message = null;
      BundleLocator locator = new BundleLocator();
      ResourceBundle bundle = locator.locateBundle(
                              pageContext.getRequest(), base);
      if(bundle != null) {
         bundles.addBundle(bundle, base);
         message = bundle.getString(key);
      }
      return message;
   }

   // prepare this tag handler for reuse

   public void release() {
      base = key = null;
      locale = null;
      params = null;
   }
}
